from pathlib import Path
from django.db.migrations.loader import MigrationLoader
from django.db import connection
from django.conf import settings

def get_app_base_dir(app_name):
    return Path(settings.BASE_DIR) / app_name

def get_migration_file_path(app_name, migration_name):
    return get_app_base_dir(app_name) / "migrations" / f"{migration_name}.py"

def get_migration_tracking_file_path(app_name):
    """Returns the path to the migration tracking file for a given app."""
    return get_app_base_dir(app_name) / "latest_migration.txt"

def store_latest_migration(app_name, migration_name):
    """Stores the latest migration name in a tracking file within the app."""
    tracking_file = get_migration_tracking_file_path(app_name)

    if not tracking_file.exists():
        return

    with tracking_file.open("w") as file:
        file.write(migration_name)

def get_latest_migration(app_name):
    """Retrieves the latest migration name from the tracking file."""
    tracking_file = get_migration_tracking_file_path(app_name)

    if not tracking_file.exists():
        return None

    with tracking_file.open("r") as file:
        return file.read()

def update_latest_migration(app_name):
    """Updates the latest migration for an app by checking the database."""
    loader = MigrationLoader(connection)
    graph = loader.graph.leaf_nodes(app_name)

    if not graph:
        return

    latest_migration = graph[-1][1]
    store_latest_migration(app_name, latest_migration)


def update_migration_dependencies(app_name, new_latest_migration, leaf_nodes, migration_file_path):
    """Updates migration dependencies to resolve conflicts."""
    dependencies = []

    for leaf in leaf_nodes:
        if leaf[1] == new_latest_migration:
            continue
        dependencies.append(f"        (\"{app_name}\", \"{leaf[1]}\"),\n")

    if not dependencies:
        return

    with migration_file_path.open("r+") as f:
        content = f.readlines()
        updated_content = []
        dependency_section = False

        for line in content:
            updated_content.append(line)

            if "dependencies = [" in line:
                dependency_section = True
                existing_dependencies = set()
            elif dependency_section:
                if line.strip().startswith("]"):
                    for dep in dependencies:
                        if dep.strip() not in existing_dependencies:
                            updated_content.insert(-1, dep)
                    dependency_section = False
                else:
                    existing_dependencies.add(line.strip())

        f.seek(0)
        f.writelines(updated_content)
        f.truncate()